 #!/usr/bin/env python
 ###########################################################################
 #                       osbackend.py module
 #                   part of the legacyconfig package
 #                       -------------------
 #   This module contains a collection of classes capable of creating menus
 #   and kde-style config files on different unix/linux platforms.  It
 #   also includes a function that will detect which class is appropriate
 #   and return it. The reason this is needed is primarily to deal with
 #   Linux distributions like Mandrake that do things like this in a
 #   nonstandard way.
 #
 #   To add support for a new OS/distribution to this module is simple:
 #   simply decide which supported OS your OS most resembles, then create
 #   a derived class that implements the difference.  Then add detection
 #   code to the factory function defined at the beginning of the file.
 #
 #   begin                : Tue May 21 2002
 #   copyright            : (C) 2002 by Jonathon Sim
 #   email                : jsim@free.net.nz
 #   url                  : http://jonsim.kdedevelopers.net
 ###########################################################################

 ###########################################################################
 #                                                                         #
 #   This program is free software; you can redistribute it and/or modify  #
 #   it under the terms of the GNU General Public License as published by  #
 #   the Free Software Foundation; either version 2 of the License, or     #
 #   (at your option) any later version.                                   #
 #                                                                         #
 ###########################################################################
DEBUG=0
import os, os.path, string, commands
def createOSHandler(userOrSystem):
	"Attempts to detect the OS, and return the appropriate backend class (defined below)"
	#Attempt to detect mandrake.  This is currently not too smart
	if os.path.exists("/etc/mandrake-release"):
		return MandrakeBackend(userOrSystem)
	else:
		return StandardBackEnd(userOrSystem)


############################################################################################
############################################################################################
class KDEPaths:
	"A class that wraps the KDE kde-config command, and performs a few sanity checks on its output"
	def userPath(self, resourceType):
		"Returns the kde dir for the current user"
		(status,output) = commands.getstatusoutput("kde-config  --path %s"%resourceType)
		if status:
			raise OSError, "kde-config returned error"
		if not len(output) > 0:
			raise OSError, "kde-config returned nonsense output"
		return string.split(output, ":")[0]

	def systemPath(self, resourceType):
		"Returns the system-wide config dir"
		(status,output) = commands.getstatusoutput("kde-config  --expandvars --install %s"%resourceType)
		if status:
			raise OSError, "kde-config returned error"
		if not len(output) > 0:
			raise OSError, "kde-config returned nonsense output"
		return string.split(output, ":")[0]

############################################################################################
############################################################################################
class StandardBackEnd:
	"""Base class for OS backend classes, that create menu and config definitions

This class will create menus and configuration files appropriate for a
standard KDE system.  The can be overridden to deal with the peculiarities of
whatever Unix system you want - eg on Mandrake > 7.x and on Debian, menus are
stored using a system-wide menu system that is converted to KDE.  """
	mDebug = DEBUG	#If true, debug output is enabled
	def __init__(self, userOrSystem):
		pathfinder = KDEPaths()
		if userOrSystem == "user":
			self.mConfigDir = pathfinder.userPath("config")
			self.mApplnkDir=  pathfinder.userPath("apps")
		elif userOrSystem == "system":
			self.mConfigDir = pathfinder.systemPath("config")
			self.mApplnkDir= pathfinder.systemPath("apps")
		else:
			raise NameError, """Must generate either "user" or "system" configuration"""
		if self.mDebug:
			self.dbgPrint("DEBUG => overriding detected applnk and config dirs:\n\t%s,\n\t %s"%(self.mApplnkDir,self.mConfigDir) )
			self.mApplnkDir="testapplnk"
			self.mConfigDir = "testconfig"
		if not os.path.exists(self.mApplnkDir):
			os.makedirs(self.mApplnkDir)
		if not os.path.exists(self.mConfigDir):
			os.makedirs(self.mConfigDir)
		self.mBaseSection="Settings/System/Aministration"


	def makeMenu(self, menuLocation, kcmName, executable, icon):
		"""Creates a menu entry, with the specified location within the menu
		structure, the specified kcontrol name, pointing to the specified
		executable"""

		desktopname = reduce(os.path.join, [self.mApplnkDir, self.mBaseSection, os.path.dirname(menuLocation), kcmName]) + ".desktop"
		self.dbgPrint("\tCreating .desktop file: %s"%desktopname)
		desktopfd = open(desktopname,"w" )
		desktopfd.write( self.desktopContents(os.path.basename(menuLocation), kcmName, icon))
		desktopfd.close()

	def removeMenu(self, menuLocation, kcmName):
		"Removes a menu entry"
		desktopname = reduce(os.path.join, [self.mApplnkDir, self.mBaseSection, os.path.dirname(menuLocation), kcmName]) + ".desktop"
		try:
			os.remove(desktopname)
		except OSError: pass	#If the file doesnt exist
	def makeConfig(self, kcmName, windowTitle, cmdLine):
		"""Creates the legacyconfig configuration file, from a windowtitle and command
		line"""
		configname = os.path.join(self.mConfigDir , kcmName) + "rc"
		self.dbgPrint("\tCreating config file: %s"%configname)
		configfd = open(configname,"w")
		configfd.write( self.configContents(windowTitle , cmdLine)  )
		configfd.close

	def createSections(self,  sections):
		"Creates the dir tree structure"
		for dir in sections:
			try : os.makedirs(reduce( os.path.join,[self.mApplnkDir,self.mBaseSection, dir[0] ]))
			except OSError: pass	#If dir already exists
			desktopfd = open( reduce( os.path.join,[self.mApplnkDir,self.mBaseSection, dir[0],".directory"]),"w")
			desktopfd.write("""[Desktop Entry]
	Name=%s
	Icon=%s
	Type=Directory
	"""%(dir[0],dir[1]) )
			desktopfd.close()

	def isAppInstalled(self, filename):
		"true if the specified application is installed"
		if os.path.isabs(filename):	#If its an absolute filename, does file exist?
			if os.path.exists(filename):
				return 1
			else:
				return 0
		else:	#Otherwise, does file exist on current path?
			for pathname in string.split(os.getenv("PATH"),":"):
				if os.path.exists( os.path.join(pathname, filename)):
					return 1
		return 0



	def close(self):
		""""Essentially a destructor (python does not support destructors) - performs any final
		operations needed to complete the installation"""
		pass

####################Utility functions########################
#These functions may be useful for implementing base classes
	def dbgPrint(self, debugString):
		"Prints a debugging message, if mDebug is true"
		if self.mDebug:
			print debugString

	def desktopContents(self,name,kcmname,icon):
		"Returns the contents of a kde .desktop file"
		return """[Desktop Entry]
Name=%s
Comment=%s
Exec=kcmshell %s
Terminal=false
Icon=%s
X-KDE-ModuleType=Library
X-KDE-Library=legacyconfig
X-KDE-FactoryName=KLegacyConfig
Type=Application
	""" %(name,name,kcmname,icon)

	def configContents(self, title, cmdline):
		"Returns the contents of a standard legacyconfig config file"
		return """[LegacyApp]
StrictTitleMatch=false
ChildTitle=%s
ChildCmdLine=%s"""%(title, cmdline)

############################################################################################
############################################################################################
class DebianStyleBackEnd(StandardBackEnd):
	"Implements debian/mandrake style menus, with standard configs"
	mMenuFileBasename = "legacyconfig"
	mPackageName = "klegacyconfig"
	def __init__(self, userOrSystem):
		StandardBackEnd.__init__(self, userOrSystem)
		if userOrSystem == "user":
			self.mDebianMenuFile = os.path.join( os.path.expanduser( "~/.menu"),self.mMenuFileBasename )
		if userOrSystem == "system":
			self.mDebianMenuFile = os.path.join( "/usr/lib/menu",self.mMenuFileBasename )
		if self.mDebug:
			self.dbgPrint("DEBUG => overriding detected menu file:"+self.mDebianMenuFile )
			self.mDebianMenuFile="legacyconfig"
		try:
			os.remove(self.mDebianMenuFile)
		except OSError: pass


	def makeMenu(self, menuLocation, kcmName, executable, icon):
		"Makes a debian-style menu"
		self.dbgPrint("\tCreating debian-style menu into  %s"%self.mDebianMenuFile)
		section = os.path.join(self.mBaseSection, os.path.dirname(menuLocation))
		contents = self.debianMenuContents( os.path.basename(menuLocation), icon, os.path.basename(menuLocation),\
							 kcmName, section, self.mPackageName)
		menufid = open(self.mDebianMenuFile, "a")
		menufid.write(contents)
		menufid.close()
	def removeMenu(self, menuLocation, kcmName):
		"There is no need to remove a menu in the debian style system"
		pass

	def createSections(self, sections):
		outstring = """?package(%s): needs="kde" section="%s" title="%s" icon="drakconf" \n"""%(self.mPackageName, os.path.dirname(self.mBaseSection), os.path.basename(self.mBaseSection))
		for dir in sections:
			outstring += """?package(%s): needs="kde" section="%s" title="%s" icon="%s" \n"""%(self.mPackageName, self.mBaseSection, dir[0], dir[1])
		menufid = open(self.mDebianMenuFile, "a")
		menufid.write(outstring)
		menufid.close()

	def close(self):
		"Updates the menus"
		if self.mDebug:
			self.dbgPrint("""DEBUG => overriding "update-menus" """)
		else:
			print "\n\tGenerating KDE menus (in the background)"
			os.system("update-menus")
	####################Utility functions########################
	#These functions may be useful for implementing base classes
	def debianMenuContents(self, name, icon, comment, kcmname,section,filepackage) :
		"Creates a debian style menu entry"
		return """?package(%s): needs="kde" kde_filename="%s" section="%s" title="%s" icon="%s" command="kcmshell %s" longtitle="%s" kde_opt="\\\\nX-KDE-ModuleType=Library\\\\nX-KDE-Library=legacyconfig\\\\nX-KDE-FactoryName=KLegacyConfig"\n """ % (filepackage,kcmname,section,name,icon,kcmname,name)



class MandrakeBackend(DebianStyleBackEnd):
	"For now this is identical to the debian backend"
	def __init__(self, userOrSystem):
		DebianStyleBackEnd.__init__(self, userOrSystem)
		self.mBaseSection = "Configuration/KDE/System/Administration"
